﻿Public Class Form1

    ' Arm Bild von http://bergasms.com/SlotMachine/full_handle2.png
    ' Sound von hier http://www.mediacollege.com/downloads/sound-effects/money/
    ' Icons von http://images.all-free-download.com/images/graphiclarge/slot_machine_icons_310830.jpg
    ' Background http://cdn.kidscreen.com/wp/wp-content/uploads/2011/06/Slot-Machine.jpg?3e2018

    Private Const ROTATION_DELAY = 100

    Private Const DISPLAYED_TILES_PER_WHEEL = 3

    Private ReadOnly WheelFlags As List(Of Boolean)

    Private ReadOnly WheelCache As List(Of FlowLayoutPanel)

    Private IsMouseDown As Boolean

    Public ReadOnly Property Running As Boolean
        Get
            Return WheelFlags.Any(Function(x) x = True)
        End Get
    End Property

    Sub New()
        InitializeComponent()
        WheelFlags = New List(Of Boolean) From {False, False, False}
        WheelCache = New List(Of FlowLayoutPanel) From {flpLeftWheel, flpMiddleWheel, flpRightWheel}
    End Sub

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        LoadAllWheelTiles()
    End Sub

    Private Sub LoadAllWheelTiles()
        For i = 1 To 3
            LoadWheelTiles(i)
        Next
    End Sub

    Private Sub LoadWheelTiles(nr As Byte)
        Dim wheel = WheelCache(nr - 1)
        For i = 1 To 9
            Dim tile = CreateWheelTile(i)
            wheel.Controls.Add(tile)
        Next
    End Sub

    Private Function CreateWheelTile(nr As Byte) As PictureBox
        Dim pb = New PictureBox()
        With pb
            .Width = flpLeftWheel.Width
            .Height = flpLeftWheel.Height \ DISPLAYED_TILES_PER_WHEEL
            .SizeMode = PictureBoxSizeMode.StretchImage
            .Margin = New Padding(0)
            .Image = CType(My.Resources.ResourceManager.GetObject($"ic{nr}"), Bitmap)
            .Tag = nr
        End With
        Return pb
    End Function

    Private Sub pbArm_MouseDown(sender As Object, e As MouseEventArgs) Handles pbArm.MouseDown
        IsMouseDown = True
    End Sub

    Private Sub pbArm_MouseUp(sender As Object, e As MouseEventArgs) Handles pbArm.MouseUp
        IsMouseDown = False
    End Sub

    Private Async Sub pbArm_MouseMove(sender As Object, e As EventArgs) Handles pbArm.MouseMove
        If Running Then
            Return
        End If
        If Not IsMouseDown Then
            Return
        End If
        Dim mousePosDifference = pbArm.PointToClient(MousePosition).Y
        Await AnimateArmAsync(mousePosDifference)
        Dim triggerPosReached = mousePosDifference >= 271
        If triggerPosReached Then
            StartRound()
        End If
    End Sub

    Private Async Function AnimateArmAsync(mousePosDifference As Integer) As Task
        Select Case mousePosDifference
            Case 0 To 30 : pbArm.Image = My.Resources.arm_frame_1
            Case 31 To 60 : pbArm.Image = My.Resources.arm_frame_2
            Case 61 To 90 : pbArm.Image = My.Resources.arm_frame_3
            Case 91 To 120 : pbArm.Image = My.Resources.arm_frame_4
            Case 121 To 150 : pbArm.Image = My.Resources.arm_frame_5
            Case 151 To 180 : pbArm.Image = My.Resources.arm_frame_6
            Case 181 To 210 : pbArm.Image = My.Resources.arm_frame_7
            Case 211 To 240 : pbArm.Image = My.Resources.arm_frame_8
            Case 241 To 270 : pbArm.Image = My.Resources.arm_frame_9
            Case 271 To 300
                pbArm.Image = My.Resources.arm_frame_10
                Await DelayArmPopupAsync()
                pbArm.Image = My.Resources.arm_frame_1
        End Select
    End Function

    Private Function DelayArmPopupAsync() As Task
        Return Task.Delay(300)
    End Function

    Private Sub StartRound()
        PlayStartSound()
        SpinWheels()
    End Sub

    Private Sub PlayStartSound()
        My.Computer.Audio.Play(My.Resources.cash_sound, AudioPlayMode.Background)
    End Sub

    Private Sub SpinWheels()
        For i = 1 To 3
            SpinWheel(i)
        Next
    End Sub

    Private Async Sub SpinWheel(nr As Byte)
        Dim buttonFlagIndex = nr - 1
        WheelFlags(buttonFlagIndex) = True
        While WheelFlags(buttonFlagIndex)
            ReInsertControl(WheelCache(buttonFlagIndex))
            Await LetRotateABit()
        End While
    End Sub

    Private Sub ReInsertControl(flp As FlowLayoutPanel)
        Dim c = flp.Controls(0)
        flp.Controls.Remove(c)
        flp.Controls.Add(c)
    End Sub

    Private Function LetRotateABit() As Task
        Return Task.Delay(ROTATION_DELAY)
    End Function

    Private Sub StopButton_MouseDown(sender As Object, e As MouseEventArgs) Handles pbStop1.MouseDown, pbStop2.MouseDown, pbStop3.MouseDown
        If Not Running Then
            Return
        End If
        Dim pb = CType(sender, PictureBox)
        pb.Image = My.Resources.button_pressed
    End Sub

    Private Sub StopButton_MouseUp(sender As Object, e As MouseEventArgs) Handles pbStop1.MouseUp, pbStop2.MouseUp, pbStop3.MouseUp
        If Not Running Then
            Return
        End If

        Dim clickedPictureBox = CType(sender, PictureBox)
        ResetButtonImage(clickedPictureBox)

        Dim clickedButtonNr = GetButtonNrByClickedPictureBox(clickedPictureBox)
        StopWheel(clickedButtonNr)

        CheckGameEnded()
    End Sub

    Private Sub ResetButtonImage(pb As PictureBox)
        pb.Image = My.Resources.button_normal
    End Sub

    Private Function GetButtonNrByClickedPictureBox(pb As PictureBox) As Byte
        Dim lastCharacterOfPictureBoxName = pb.Name.Substring(pb.Name.Length - 1, 1)
        Dim clickedButtonNr = Convert.ToByte(lastCharacterOfPictureBoxName)
        Return clickedButtonNr
    End Function

    Private Sub CheckGameEnded()
        If Running Then
            Return
        End If
        Dim tileMatrix = GetTileMatrix()
        Dim args = New GameEndedEventArgs(tileMatrix)
        OnGameEnded(args)
    End Sub

    Private Function GetTileMatrix() As List(Of List(Of PictureBox))
        Dim list = New List(Of List(Of PictureBox))
        For i = 1 To 3
            Dim iconList = GetTileListForWheel(i)
            list.Add(iconList)
        Next
        Return list
    End Function

    Private Function GetTileListForWheel(wheelNr As Byte) As List(Of PictureBox)
        Dim list = New List(Of PictureBox)
        Dim wheelIndex = wheelNr - 1
        Dim flpWheel = WheelCache(wheelIndex)
        For i = 0 To 2
            Dim pictureBox = flpWheel.Controls(i)
            list.Add(pictureBox)
        Next
        Return list
    End Function

    Protected Sub OnGameEnded(e As GameEndedEventArgs)
        RaiseEvent GameEnded(Me, e)
    End Sub

    Private Sub Me_GameEnded(sender As Object, e As GameEndedEventArgs) Handles Me.GameEnded
        ' CalculatePoints?
        ' Show Message?
        ' Blink success tiles?
    End Sub

    Private Sub pbExit_Click(sender As Object, e As EventArgs) Handles pbExit.Click
        Close()
    End Sub

    Private Sub Form1_FormClosing(sender As Object, e As FormClosingEventArgs) Handles Me.FormClosing
        StopWheels()
    End Sub

    Private Sub StopWheels()
        For i = 1 To 3
            StopWheel(i)
        Next
    End Sub

    Private Sub StopWheel(nr As Byte)
        Dim wheelFlagIndex = nr - 1
        WheelFlags(wheelFlagIndex) = False
    End Sub

    Public Event GameEnded As EventHandler(Of GameEndedEventArgs)

End Class